/**
 * Title :        XLogo
 * Description :  XLogo is an interpreter for the Logo 
 * 						programming language
 * @author Loïc Le Coq
 */
package xlogo.utils;
import java.awt.Container;
import java.awt.Font;
import java.awt.Image;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.MediaTracker;
import java.io.BufferedReader;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.net.URL;

import xlogo.kernel.Affichage;



public class Utils {
	public static Image dimensionne_image(String nom,Component jf){
		Image image=null;
		image= Toolkit.getDefaultToolkit().getImage(Utils.class.getResource(nom));
		MediaTracker tracker=new MediaTracker(jf);
		tracker.addImage(image,0);
		try{tracker.waitForID(0);}
		catch(InterruptedException e1){}
		double largeur_ecran=Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int largeur=image.getWidth(jf); 
		int hauteur=image.getHeight(jf);
		// On fait attention à la résolution de l'utilisateur
		double facteur = largeur_ecran/1024.0; //les images sont prévues pour 1024x768
		if ((int)(facteur+0.001)!=1){
			image=image.getScaledInstance((int)(facteur*largeur),(int)(facteur*hauteur),Image.SCALE_SMOOTH);
			tracker=new MediaTracker(jf);
			tracker.addImage(image,0);
			try{tracker.waitForID(0);}
			catch(InterruptedException e1){}
		}
		return image;
	  }
	public static void recursivelySetFonts(Component comp, Font font) {
	    comp.setFont(font);
	    if (comp instanceof Container) {
	      Container cont = (Container) comp;
	      for(int j=0, ub=cont.getComponentCount(); j<ub; ++j)
			recursivelySetFonts(cont.getComponent(j), font);
	    }
	}
	public static String rajoute_backslash(String st){
		StringBuffer tampon=new StringBuffer();
		for(int j=0;j<st.length();j++){
			char c=st.charAt(j);
			if (c=='\\') tampon.append("\\\\");
			else if (c==' ') tampon.append("\\e");
			else if ("()[]#".indexOf(c)!=-1) tampon.append("\\"+c);
			else tampon.append(c);
		}
		return(new String(tampon));
	  }
	public static String SortieTexte(String chaine){ // Enlève les backslash
		StringBuffer buffer=new StringBuffer();
		boolean backslash=false;
		for (int j=0;j<chaine.length();j++){
			char c=chaine.charAt(j);
			if (backslash) {
				if (c=='e') buffer.append(' ');
//				else if (c=='\\') buffer.append('\\');
				else if (c=='n') buffer.append("\n");
				else if(c=='v') buffer.append("");
				else if("[]()#\\".indexOf(c)>-1) buffer.append(c);
				backslash=false;
			}
			else {
				if (c=='\\') backslash=true;
				else buffer.append(c);
			}
		}
		return new String(buffer);
	}
	public static StringBuffer decoupe(String st) { //Supprime les enchainements d'espaces inutiles et modifie le caractère \ en \\ 
		StringBuffer buffer = new StringBuffer();
		boolean espace=false;
		boolean backslash=false;
		boolean mot=false;
		int crochet_liste=0;
		boolean variable=false;
		boolean execution_lancee=Affichage.execution_lancee;
		for(int i=0;i<st.length();i++){
			char c=st.charAt(i);
			if (c==' ') {
				if (!espace&&buffer.length()!=0) {
					if (backslash) buffer.append("\\e");
					else {
						buffer.append(c);
						espace=true;
						mot=false;
						variable=false;
					}
					backslash=false;
				}
			}
			else if(c=='\\'&&!backslash) {
				espace=false;
				backslash=true;
			}
			else if(c=='\"'){
				if (espace&&crochet_liste<=0){
					mot=true;
				}
				buffer.append(c);
				espace=false;
				backslash=false;
			}
			else if (c==':'){
				if (espace&&crochet_liste<=0){
					variable=true;
				}
				buffer.append(c);
				espace=false;
				backslash=false;
			}
			else if (c=='['||c==']'||c=='('||c==')'){
				//Modifications apportées
				if (backslash) {
					buffer.append("\\"+c);
					backslash=false;
				}
				else {
					if (c=='[') crochet_liste++;
					else if (c==']') crochet_liste--;
					if (espace||buffer.length()==0) {buffer.append(c+" ");espace=true;}
					else {
						buffer.append(" "+c+" ");
						espace=true;
					}
				}
			}
			else if (c=='+'||c=='-'||c=='*'||c=='/'||c=='='||c=='<'||c=='>'||c=='&'||c=='|'){
				// à modifier (test + fin)
				if (mot||crochet_liste>0) buffer.append(c);
				else { 
					if (espace) buffer.append(c+" ");
					else {
						espace=true;
						buffer.append(" "+c+" ");
					}
				}
			}
			else{
				if (backslash){
					if (c=='n')	buffer.append("\\n");
					else if (c=='\\') buffer.append("\\\\"); 
					else if (c=='v'&& execution_lancee) buffer.append("\"");
					else if(c=='e'&& execution_lancee) buffer.append("\\e");
					else if (c=='#') buffer.append("\\#");
					else { 
						buffer.append(c);
					}
				}
				else {
					buffer.append(c);	
				}
				backslash=false;
				espace=false;
			}
		}
		//System.out.println(buffer);
		return (buffer);
	}
	
	
	
	public static String specialCharacterXML(String st){
		st=st.replaceAll("&","&amp;");
		st=st.replaceAll("<","&lt;");
		st=st.replaceAll("\"","&quot;");
		st=st.replaceAll(">","&gt;");
		st=st.replaceAll("'","&apos;");
		
		return st;
	}
	public static String readLogoFile(String path) throws IOException{
		String txt="";
		// Try to read the lgo format for XLogo <= 0.9.22
		try{
            RandomAccessFile f=new RandomAccessFile(path,"r");
            f.seek(0);
            txt=f.readUTF();
            f.close();
          }
          catch(FileNotFoundException e1){
          	// tentative fichier réseau 
          	try{
          		URL url =new java.net.URL(path);
    			StringBuffer sb=new StringBuffer();
              	 java.io.InputStream fr = url.openStream();
                  InputStreamReader isr = new  InputStreamReader(fr,  "UTF8");
                  BufferedReader brd=new BufferedReader(isr);
                  while (brd.ready()){
                  	sb.append(brd.readLine());
                  	sb.append("\n");
                  }
                  txt=new String(sb);
          	}
          	catch( java.net.MalformedURLException e){
          		System.out.println("Fichier non trouvé");	
          	}
          }
          catch(java.io.EOFException e2){ 
          	// New format for XLogo >=0.923
          	// Encoded with UTF-8
    		StringBuffer sb=new StringBuffer();
          	 FileInputStream fr = new FileInputStream(path);
              InputStreamReader isr = new  InputStreamReader(fr,  "UTF8");
              BufferedReader brd=new BufferedReader(isr);
              while (brd.ready()){
              	sb.append(brd.readLine());
              	sb.append("\n");
              }
              txt=new String(sb);
            }
          catch(Exception e){e.printStackTrace();}
          return txt;
	}
	public static void writeLogoFile(String path,String txt) throws IOException{
		try{
			FileOutputStream f = new FileOutputStream(path);
			BufferedOutputStream b = new BufferedOutputStream(f);	
			OutputStreamWriter osw = new  OutputStreamWriter(b,  "UTF8");
	        osw.write(txt);
	        osw.close();
	        b.close();
	        f.close();

		}
		catch(FileNotFoundException e1){e1.printStackTrace();}
	}
}
